@@ -99,8 +99,9 @@ namespace Astroid {
           g_mime_object_get_header_list (GMIME_OBJECT(message)),
           "In-Reply-To");
     } else {
+      ustring tmp = "<" + inreplyto + ">";
       g_mime_object_set_header (GMIME_OBJECT(message), "In-Reply-To",
-          inreplyto.c_str(), NULL);
+          tmp.c_str(), NULL);
     }
   }
 
@@ -156,7 +157,7 @@ namespace Astroid {
         sf << s.rdbuf ();
         s.close ();
         if (account->signature_separate) {
-    md_body_content += "-- \n";
+          md_body_content += "-- \n";
         }
         md_body_content += sf.str ();
       }
@@ -227,6 +228,7 @@ namespace Astroid {
           contentStream = g_mime_stream_mem_new_with_buffer(_html.c_str(), _html.size());
         }
 
+        g_spawn_close_pid (pid);
       } catch (Glib::SpawnError &ex) {
         LOG (error) << "cm: md: failed to spawn markdown processor: " << ex.what ();
 
@@ -260,7 +262,7 @@ namespace Astroid {
 
   void ComposeMessage::load_message (ustring _mid, ustring fname) {
     set_id (_mid);
-    Message msg (_mid, fname);
+    UnprocessedMessage msg (_mid, fname);
 
     Account * from = astroid->accounts->get_account_for_address (msg.sender);
     if (from == NULL) {
@@ -283,23 +285,13 @@ namespace Astroid {
 
     set_subject (msg.subject);
 
-    body << msg.viewable_text (false);
-
+    body << msg.plain_text (false);
   }
 
   void ComposeMessage::finalize () {
     /* make message ready to be sent */
     LOG (debug) << "cm: finalize..";
 
-    /* again: ripped more or less from ner */
-
-    /*
-    FILE * file = fopen(_messageFile.c_str(), "r");
-    GMimeStream * stream = g_mime_stream_file_new(file);
-    GMimeParser * parser = g_mime_parser_new_with_stream(stream);
-    GMimeMessage * message = g_mime_parser_construct_message(parser);
-    */
-
     /* set user agent */
     ustring ua = "";
 
@@ -671,7 +663,7 @@ namespace Astroid {
     /* add to notmuch with sent tag (on main GUI thread) */
     if (!dryrun && message_sent_result && account->save_sent) {
       astroid->actions->doit (refptr<Action> (
-            new AddSentMessage (save_to.c_str (), account->additional_sent_tags)));
+            new AddSentMessage (save_to.c_str (), account->additional_sent_tags, inreplyto)));
       LOG (info) << "cm: sent message added to db.";
     }
 
@@ -732,6 +724,18 @@ namespace Astroid {
     LOG (debug) << "cm: wrote file: " << fname;
   }
 
+  void ComposeMessage::write (GMimeStream * stream) {
+    g_object_ref (stream);
+
+    g_mime_object_write_to_stream (GMIME_OBJECT(message), g_mime_format_options_get_default (), stream);
+    g_mime_stream_flush (stream);
+    g_mime_stream_seek (stream, 0, GMIME_STREAM_SEEK_SET);
+
+    g_object_unref(stream);
+
+    LOG (debug) << "cm: wrote to stream.";
+  }
+
   ComposeMessage::Attachment::Attachment () {
   }
 
@@ -771,7 +775,7 @@ namespace Astroid {
 
     if (content_type == "message/rfc822") {
       LOG (debug) << "cm: attachment is mime message.";
-      message = refptr<Message> (new Message(fname.c_str ()));
+      message = refptr<Message> (new UnprocessedMessage(fname.c_str ()));
 
       is_mime_message = true;
 
@@ -813,7 +817,7 @@ namespace Astroid {
       content_type = "message/rfc822";
       is_mime_message = true;
 
-      message = refptr<Message> (new Message(GMIME_MESSAGE(c->mime_object)));
+      message = refptr<Message> (new UnprocessedMessage(GMIME_MESSAGE(c->mime_object)));
 
       g_object_ref (c->mime_object); // should be cleaned by Message : Glib::Object
 
