@@ -156,7 +156,12 @@ namespace Astroid {
   ptree Config::setup_default_config (bool initial) {
     ptree default_config;
     default_config.put ("astroid.config.version", CONFIG_VERSION);
+
     std::string nm_cfg = path(std_paths.home / path (".notmuch-config")).string();
+    char* nm_env = getenv("NOTMUCH_CONFIG");
+    if (nm_env != NULL) {
+      nm_cfg.assign(nm_env, strlen(nm_env));
+    }
     default_config.put ("astroid.notmuch_config" , nm_cfg);
 
     default_config.put ("astroid.debug.dryrun_sending", false);
@@ -189,6 +194,8 @@ namespace Astroid {
     default_config.put ("general.time.same_year", "%b %-e");
     default_config.put ("general.time.diff_year", "%x");
 
+    default_config.put ("general.tagbar_move", "tag");
+
     /* thread index cell theme */
     default_config.put ("thread_index.cell.font_description", "default");
     default_config.put ("thread_index.cell.line_spacing", 2);
@@ -226,6 +233,9 @@ namespace Astroid {
     default_config.put ("editor.attachment_directory", "~");
 
     default_config.put ("editor.markdown_processor", "cmark");
+    default_config.put ("editor.markdown_on", false); // default
+
+    default_config.put ("mail.reply.quote_processor", "w3m -dump -T text/html"); // e.g. lynx -dump
 
     /* mail composition */
     default_config.put ("mail.reply.quote_line", "Excerpts from %1's message of %2:"); // %1 = author, %2 = pretty_verbose_date
