@@ -11,11 +11,15 @@
 # include "message_thread.hh"
 # include "chunk.hh"
 # include "utils/utils.hh"
+# include "utils/cmd.hh"
 # include "utils/date_utils.hh"
 # include "utils/address.hh"
 # include "utils/ustring_utils.hh"
 # include "utils/vector_utils.hh"
 # include "actions/action_manager.hh"
+# ifndef DISABLE_PLUGINS
+  # include "plugin/manager.hh"
+# endif
 
 using namespace std;
 using namespace boost::filesystem;
@@ -87,7 +91,7 @@ namespace Astroid {
     tags = nmmsg->tags;
   }
 
-  Message::Message (GMimeMessage * _msg) {
+  Message::Message (GMimeMessage * _msg) : Message () {
     LOG (info) << "msg: loading message from GMimeMessage.";
     in_notmuch = false;
     has_file   = false;
@@ -96,6 +100,23 @@ namespace Astroid {
     load_message (_msg);
   }
 
+  Message::Message (GMimeStream * s) : Message () {
+    LOG (info) << "msg: loading message from GMimeStream.";
+    in_notmuch = false;
+    has_file   = false;
+    missing_content = false;
+
+    g_object_ref (s);
+    GMimeParser   * parser  = g_mime_parser_new_with_stream (s);
+    GMimeMessage * _message = g_mime_parser_construct_message (parser, g_mime_parser_options_get_default ());
+
+    load_message (_message);
+
+    g_object_unref (_message);
+    g_object_unref (s);
+    g_object_unref (parser);
+  }
+
   Message::~Message () {
     LOG (debug) << "ms: deconstruct";
     if (message) g_object_unref (message);
@@ -170,30 +191,37 @@ namespace Astroid {
       return;
 
     } else {
-      GError *err = NULL; (void) (err); // not used in GMime 2.
-      GMimeStream   * stream  = g_mime_stream_file_open (fname.c_str(), "r", &err);
-      g_mime_stream_file_set_owner (GMIME_STREAM_FILE(stream), TRUE);
+      GMimeStream * stream = NULL;
+# ifndef DISABLE_PLUGINS
+      if (process) {
+        stream = astroid->plugin_manager->astroid_extension->process (fname.c_str());
+      }
+# endif
       if (stream == NULL) {
-        LOG (error) << "failed to open file: " << fname << " (unspecified error)";
+        GError *err = NULL; (void) (err); // not used in GMime 2.
+        stream  = g_mime_stream_file_open (fname.c_str(), "r", &err);
+        g_mime_stream_file_set_owner (GMIME_STREAM_FILE(stream), TRUE);
+        if (stream == NULL) {
+          LOG (error) << "failed to open file: " << fname << " (unspecified error)";
 
-        has_file = false;
-        missing_content = true;
+          has_file = false;
+          missing_content = true;
 
-        if (in_notmuch) {
-          LOG (warn) << "loading cache for missing file from notmuch";
-          load_notmuch_cache ();
-        } else {
-          LOG (error) << "tried to open disk file, but failed, message is not in database either.";
-          string error_s = "failed to open file: " + fname;
-          throw message_error (error_s.c_str());
+          if (in_notmuch) {
+            LOG (warn) << "loading cache for missing file from notmuch";
+            load_notmuch_cache ();
+          } else {
+            LOG (error) << "tried to open disk file, but failed, message is not in database either.";
+            string error_s = "failed to open file: " + fname;
+            throw message_error (error_s.c_str());
+          }
+          return;
         }
-        return;
       }
 
       GMimeParser   * parser  = g_mime_parser_new_with_stream (stream);
       GMimeMessage * _message = g_mime_parser_construct_message (parser, g_mime_parser_options_get_default ());
       load_message (_message);
-
       g_object_unref (_message); // is reffed in load_message
       g_object_unref (stream); // reffed from parser
       g_object_unref (parser); // reffed from message
@@ -292,21 +320,12 @@ namespace Astroid {
     root = refptr<Chunk>(new Chunk (g_mime_message_get_mime_part (message)));
   }
 
-  ustring Message::viewable_text (bool html, bool fallback_html) {
-    /* build message body:
-     * html:      output html (using gmimes html filter)
-     *
-     */
-
+  ustring Message::plain_text (bool fallback_html) {
     if (missing_content) {
       LOG (warn) << "message: missing content, no text.";
       return "";
     }
 
-    if (html && fallback_html) {
-      throw logic_error ("message: html implies fallback_html");
-    }
-
     ustring body;
 
     function< void (refptr<Chunk>) > app_body =
@@ -316,13 +335,13 @@ namespace Astroid {
       bool use = false;
 
       if (c->siblings.size() >= 1) {
-        if (c->preferred) {
+        if (c->is_content_type ("text", "plain") || fallback_html) {
           use = true;
         } else {
           /* check if there are any other preferred */
           if (all_of (c->siblings.begin (),
                       c->siblings.end (),
-                      [](refptr<Chunk> c) { return (!c->preferred); })) {
+                      [fallback_html](refptr<Chunk> c) { return !(c->is_content_type ("text", "plain") || fallback_html); })) {
             use = true;
           } else {
             use = false;
@@ -333,8 +352,70 @@ namespace Astroid {
       }
 
       if (use) {
-        if (c->viewable && (c->preferred || html || fallback_html)) {
-          body += c->viewable_text (html);
+        if (c->viewable && (c->is_content_type ("text", "plain") || fallback_html)) {
+          /* will output html if HTML part */
+          body += c->viewable_text (false);
+        }
+
+        for_each (c->kids.begin(),
+                  c->kids.end (),
+                  app_body);
+      }
+    };
+
+    app_body (root);
+
+    return body;
+  }
+
+  ustring Message::quote () {
+    if (missing_content) {
+      LOG (warn) << "message: missing content, no text.";
+      return "";
+    }
+
+    ustring body;
+
+    function< void (refptr<Chunk>) > app_body =
+      [&] (refptr<Chunk> c)
+    {
+      /* check if we're the preferred sibling */
+      bool use = false;
+
+      if (c->siblings.size() >= 1) {
+        if (c->is_content_type ("text", "plain") || c->is_content_type ("text", "html")) {
+          use = true;
+        } else {
+          /* check if there are any other preferred */
+          if (all_of (c->siblings.begin (),
+                      c->siblings.end (),
+                      [](refptr<Chunk> c) { return !(c->is_content_type ("text", "plain") || c->is_content_type("text", "html")); })) {
+            use = true; // no
+          } else {
+            use = false;
+          }
+        }
+      } else {
+        use = true;
+      }
+
+      if (use) {
+        if (c->viewable && (c->is_content_type ("text", "plain") || c->is_content_type ("text", "html"))) {
+          /* will output html if HTML part */
+          if (c->is_content_type ("text", "html")) {
+            ustring quote_cmd = astroid->config ().get<string>("mail.reply.quote_processor");
+
+            if (!quote_cmd.empty()) {
+                ustring h = c->viewable_text (false);
+                ustring _stdout, _stderr;
+                Cmd::pipe (quote_cmd, h, _stdout, _stderr);
+
+                body += _stdout;
+            }
+
+          } else {
+            body += c->viewable_text (false);
+          }
         }
 
         for_each (c->kids.begin(),
@@ -856,6 +937,82 @@ namespace Astroid {
   {
   }
 
+  /**********************
+   * UnprocessedMessage *
+   **********************/
+  UnprocessedMessage::UnprocessedMessage (ustring _fname) : Message () {
+
+    LOG (info) << "msg: loading message from file: " << fname;
+    fname    = _fname;
+    has_file = true;
+    process  = false;
+    load_message_from_file (fname);
+  }
+
+  UnprocessedMessage::UnprocessedMessage (
+      ustring _mid,
+      ustring _fname)
+    : Message ()
+  {
+    mid = _mid;
+    fname = _fname;
+    process = false; // must be set before load_message_from_file
+    has_file = true;
+
+    LOG (info) << "msg: loading message from file (mid supplied): " << fname;
+    load_message_from_file (fname);
+  };
+
+  UnprocessedMessage::UnprocessedMessage (GMimeStream * s) : Message () {
+    LOG (info) << "msg: loading message from GMimeStream.";
+    in_notmuch = false;
+    has_file   = false;
+    process    = false;
+    missing_content = false;
+
+    g_object_ref (s);
+    GMimeParser   * parser  = g_mime_parser_new_with_stream (s);
+    GMimeMessage * _message = g_mime_parser_construct_message (parser, g_mime_parser_options_get_default ());
+
+    load_message (_message);
+
+    g_object_unref (_message);
+    g_object_unref (s);
+    g_object_unref (parser);
+  }
+
+  UnprocessedMessage::UnprocessedMessage (GMimeMessage * _msg) : Message () {
+    LOG (info) << "msg: loading message from GMimeMessage.";
+    in_notmuch = false;
+    has_file   = false;
+    process    = false;
+    missing_content = false;
+
+    load_message (_msg);
+  }
+
+  UnprocessedMessage::UnprocessedMessage (notmuch_message_t *message, int _level) : Message () {
+    /* The caller must make sure the message pointer
+     * is valid and not destroyed while initializing */
+
+    mid = notmuch_message_get_message_id (message);
+    tid = notmuch_message_get_thread_id (message);
+    in_notmuch = true;
+    has_file   = true;
+    level      = _level;
+    process    = false;
+
+    nmmsg = refptr<NotmuchMessage> (new NotmuchMessage (message));
+
+    LOG (info) << "msg: loading mid: " << mid;
+
+    fname = nmmsg->filename;
+    LOG (info) << "msg: filename: " << fname;
+
+    load_message_from_file (fname);
+    tags = nmmsg->tags;
+  }
+
   /* --------
    * MessageThread
    * --------
@@ -971,6 +1128,12 @@ namespace Astroid {
     }
   }
 
+  void MessageThread::add_message (refptr<Message> m) {
+    if (!first_subject_set) set_first_subject (m->subject);
+    m->subject_is_different = subject_is_different (m->subject);
+    messages.push_back (m);
+  }
+
   std::vector<refptr<Message>> MessageThread::messages_by_time () {
     auto f = [&] (refptr<Message> a, refptr<Message> b) {
        return a->time < b->time;
