@@ -35,13 +35,15 @@ using namespace boost::filesystem;
 namespace Astroid {
   int EditMessage::edit_id = 0;
 
-  EditMessage::EditMessage (MainWindow * mw, ustring _to, ustring _from, ustring _cc, ustring _bcc) :
+  EditMessage::EditMessage (MainWindow * mw, ustring _to, ustring _from, ustring _cc, ustring _bcc, ustring _subject, ustring _body) :
     EditMessage (mw, false) { // {{{
 
     in_read = false;
     to  = _to;
     cc  = _cc;
     bcc = _bcc;
+    subject = _subject;
+    body = _body;
     if (!_from.empty ()) {
       set_from (Address (_from));
     }
@@ -282,6 +284,8 @@ namespace Astroid {
 
     reset_signature ();
 
+    switch_markdown->set_active (astroid->config ().get<bool> ("editor.markdown_on"));
+
     /* register keys {{{ */
     keys.title = "Edit mode";
     keys.register_key (Key (GDK_KEY_Return), { Key (GDK_KEY_KP_Enter) },
@@ -332,13 +336,15 @@ namespace Astroid {
         "View raw message",
         [&] (Key) {
           /* view raw source of to be sent message */
-          ComposeMessage * c = make_message ();
-          ustring tmpf = c->write_tmp ();
+          auto c = make_message ();
 
-          main_window->add_mode (new RawMessage (main_window, tmpf.c_str(), true));
-          /* tmp file deleted by RawMessage */
+          GMimeStream * m = g_mime_stream_mem_new ();
+          assert (g_mime_stream_mem_get_owner (GMIME_STREAM_MEM(m)) == true);
+          c->write (m);
 
-          delete c;
+          main_window->add_mode (new RawMessage (main_window, refptr<Message>(new UnprocessedMessage(m))));
+
+          g_object_unref (m);
 
           return true;
         });
@@ -389,7 +395,7 @@ namespace Astroid {
                         if (msg != NULL) {
                           LOG (debug) << "em: attaching: " << mid;
 
-                          refptr<Message> mmsg  = refptr<Message> (new Message (msg, 0));
+                          refptr<Message> mmsg  = refptr<Message> (new UnprocessedMessage (msg, 0));
                           add_attachment (new ComposeMessage::Attachment (mmsg));
 
                         } else {
@@ -541,6 +547,10 @@ namespace Astroid {
     }
   }
 
+  void EditMessage::pre_close () {
+    ((Mode*) thread_view)->pre_close ();
+  }
+
   void EditMessage::close (bool force) {
     if (sending_in_progress.load ()) {
       /* block closing the window while sending */
@@ -573,7 +583,7 @@ namespace Astroid {
   bool EditMessage::save_draft () {
     LOG (info) << "em: saving draft..";
     draft_saved = false;
-    ComposeMessage * c = make_draft_message ();
+    auto c = make_draft_message ();
     ustring fname;
 
     bool add_to_notmuch = false;
@@ -585,7 +595,6 @@ namespace Astroid {
       if (!is_directory(ddir)) {
         LOG (error) << "em: no draft directory specified!";
         set_warning ("draft could not be saved, no suitable draft directory for account specified.");
-        delete c;
         return false;
 
       } else {
@@ -608,7 +617,6 @@ namespace Astroid {
             new AddDraftMessage (fname)));
     }
 
-    delete c;
     draft_saved = true;
     return true;
   }
@@ -776,7 +784,7 @@ namespace Astroid {
     }
 
     /* make message */
-    ComposeMessage * c = setup_message ();
+    auto c = setup_message ();
 
     /* set account selector to from address email */
     set_from (c->account);
@@ -802,15 +810,15 @@ namespace Astroid {
     subject = c->subject;
     body = ustring(c->body.str());
 
-    ustring tmpf = c->write_tmp ();
+    GMimeStream * m = g_mime_stream_mem_new ();
+    assert (g_mime_stream_mem_get_owner (GMIME_STREAM_MEM(m)) == true);
+    c->write (m);
 
     auto msgt = refptr<MessageThread>(new MessageThread());
-    msgt->add_message (tmpf);
+    msgt->add_message (refptr<Message>(new UnprocessedMessage(m)));
     thread_view->load_message_thread (msgt);
 
-    delete c;
-
-    unlink (tmpf.c_str());
+    g_object_unref (m);
 
     in_read = false;
   }
@@ -1037,10 +1045,22 @@ namespace Astroid {
     if (attachments.empty () && !attachment_words.empty ()) {
       ustring bl = body.lowercase ();
 
+      /* strip quoted lines */
+      ostringstream nonquoted;
+      stringstream sstr (bl);
+      while (sstr.good()) {
+        string line;
+        getline (sstr, line);
+
+        if (line[0] != '>')
+          nonquoted << line << endl;
+      }
+      ustring nqb = ustring(nonquoted.str());
+
       if (any_of (attachment_words.begin (),
                   attachment_words.end (),
                   [&] (ustring w) {
-                    return bl.find (w) != string::npos;
+                    return nqb.find (w) != string::npos;
                   }))
       {
         set_warning ("Attachments have been mentioned in the message, but none are attached, do you still want to send?");
@@ -1063,21 +1083,18 @@ namespace Astroid {
 
     on_tv_ready ();
 
-    ComposeMessage * c = make_message ();
+    auto c = make_message ();
 
     if (c == NULL) return false;
 
     if (c->markdown && !c->markdown_success) {
       set_warning ("Cannot send, failed processing markdown: " + UstringUtils::replace (c->markdown_error, "\n", "<br />"));
-      delete c;
       return false;
     }
 
     if (c->encrypt || c->sign) {
       if (!c->encryption_success) {
         set_warning ("Cannot send, failed encrypting: " + UstringUtils::replace (c->encryption_error, "\n", "<br />"));
-
-        delete c;
         return false;
       }
     }
@@ -1103,7 +1120,7 @@ namespace Astroid {
 
     c->send_threaded ();
 
-    sending_message = c;
+    sending_message = std::move (c);
 
     return true;
   }
@@ -1153,15 +1170,14 @@ namespace Astroid {
     message_sending_status_icon.set (pixbuf);
     sending_in_progress.store (false);
 
+    sending_message.reset();
+
+    emit_message_sent_attempt (result_from_sender);
+
     if (result_from_sender && (astroid->config().get<bool> ("mail.close_on_success"))) {
       LOG (info) << "cm: sending successful, auto-closing window";
       close (true);
     }
-
-
-    delete sending_message;
-
-    emit_message_sent_attempt (result_from_sender);
   }
 
   void EditMessage::lock_message_after_send () {
@@ -1170,8 +1186,8 @@ namespace Astroid {
     fields_hide ();
   }
 
-  ComposeMessage * EditMessage::setup_message () {
-    ComposeMessage * c = new ComposeMessage ();
+  std::unique_ptr<ComposeMessage> EditMessage::setup_message () {
+    auto c = std::unique_ptr<ComposeMessage>(new ComposeMessage ());
 
     c->load_message (msg_id, tmpfile_path.c_str());
 
@@ -1185,7 +1201,7 @@ namespace Astroid {
     return c;
   }
 
-  void EditMessage::finalize_message (ComposeMessage * c) {
+  void EditMessage::finalize_message (std::unique_ptr<ComposeMessage> &c) {
     /* these options are not known before setup_message is done, and the
      * new account information has been applied to the editor */
     if (c->account->has_signature && switch_signature->get_active ()) {
@@ -1204,16 +1220,16 @@ namespace Astroid {
     c->finalize ();
   }
 
-  ComposeMessage * EditMessage::make_message () {
+  std::unique_ptr<ComposeMessage> EditMessage::make_message () {
     return make_message (false);
   }
 
-  ComposeMessage * EditMessage::make_draft_message () {
+  std::unique_ptr<ComposeMessage> EditMessage::make_draft_message () {
     return make_message (true);
   }
 
-  ComposeMessage * EditMessage::make_message (bool draft = false) {
-    ComposeMessage * c = setup_message ();
+  std::unique_ptr<ComposeMessage> EditMessage::make_message (bool draft = false) {
+    auto c = setup_message ();
     bool sigstate = c->account->has_signature;
 
     if (draft) {
@@ -1242,7 +1258,7 @@ namespace Astroid {
 
     if (tmpfile.fail()) {
       LOG (error) << "em: error: could not create tmpfile!";
-      throw runtime_error ("em: coult not create tmpfile!");
+      throw runtime_error ("em: could not create tmpfile!");
     }
 
     tmpfile.close ();
